/*
#  -------------------------------------------------------------------
# This file is part of Rule2Phrase.
#
# Rule2Phrase is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Rule2Phrase is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rule2Phrase.  If not, see <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------
*/
package es.ua.dlsi.hybridmt.apertiumphrasegenerator;

import es.ua.dlsi.hybridmt.schemas.interchunk.Interchunk;
import es.ua.dlsi.hybridmt.schemas.transfer.Transfer;
import java.io.BufferedWriter;
import java.io.File;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.LinkedList;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;

/**
 * Hello world!
 *
 */
public class Main
{
    public static void main( String[] args ) throws Exception
    {

        String lexformsFile=null;
        String rulesFile=null;
        String ngramsFile=null;
        int numNGramsFiles=-1;

        int ngramOrder=3;

        String splitFile=null;

        boolean generateAlignments=true;

        //Parse command line
        Options options = new Options();

        options.addOption("g", "generate",false,"Generate lexical forms");
        options.addOption("p","pretransfer_reverse",false,"Reverse pretransfer by joining lexical forms");
        options.addOption("x", "generate2level" ,  false, "Generate lexical forms using 2nd level of transfer");

        options.addOption("l", "lexforms", true, "Lexical forms file");
        options.addOption("r", "rules", true, "Apertium transfer rules file");
        options.addOption("t", "trie", true, "Ngrams file");
        options.addOption("m", "multipleNgrams",true,"Number of ngrams files");

        options.addOption("s","split_words",true,"Split words file");
        options.addOption("n","no_alignments",false,"Do not generate alignments (faster)");

        options.addOption("o","ngramOrder",true,"N-gram order");

        CommandLineParser clparser = new PosixParser();
        CommandLine line = clparser.parse( options, args );

        if(line.hasOption("g") || line.hasOption("x"))
        {

        if (line.hasOption("l"))
            lexformsFile=line.getOptionValue("l");
        if (line.hasOption("r"))
            rulesFile=line.getOptionValue("r");
        if(line.hasOption("t"))
            ngramsFile=line.getOptionValue("t");
        if(line.hasOption("m"))
            numNGramsFiles=Integer.parseInt(line.getOptionValue("m"));
        if(line.hasOption("o"))
            ngramOrder=Integer.parseInt(line.getOptionValue("o"));

        }
        else if(line.hasOption("p"))
        {
            if(line.hasOption("s"))
                splitFile=line.getOptionValue("s");
        }
        if(line.hasOption("n"))
                generateAlignments=false;

        if ( ( !line.hasOption("p") && !line.hasOption("g") && !line.hasOption("x")) || (line.hasOption("p")  && splitFile==null)  || (line.hasOption("g") && (lexformsFile==null || rulesFile==null || ngramsFile==null)) || (line.hasOption("x") && (lexformsFile==null || rulesFile==null  || ngramsFile==null ))  )
        {
            System.err.println("wrong parameters");
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp( "ApertiumPhraseGenerator", options );
            System.exit(1);
        }

        if(line.hasOption("g"))
        {
            //Loading rules
            File f = new File(rulesFile);
            JAXBContext context = JAXBContext.newInstance(Transfer.class);
            Unmarshaller u = context.createUnmarshaller();
            Transfer transfer = (Transfer) u.unmarshal(f);

            //Load lexical forms
            WordsTrie wordsTrie=new WordsTrie();
            wordsTrie.loadFromFile(new File(lexformsFile));

            //Load bigrams and trigrams
            NGramTrie nGramTrie=new NGramTrie();
            nGramTrie.loadFromFile(new File(ngramsFile));

            //Generate phrases
            PhraseGenerator phraseGenerator=new PhraseGenerator(transfer,wordsTrie);
            BufferedWriter writer = new BufferedWriter( new OutputStreamWriter(System.out,"UTF-8") );
            phraseGenerator.generatePhrases(writer, nGramTrie,ngramOrder);
            writer.close();
        }
        else if (line.hasOption("x"))
        {
            //Loading rules
            File f = new File(rulesFile);
            JAXBContext context = JAXBContext.newInstance(Interchunk.class);
            Unmarshaller u = context.createUnmarshaller();
            Interchunk interchunk = (Interchunk) u.unmarshal(f);

            List<NGramTrie> ngramTries= new LinkedList<NGramTrie>();

            if(numNGramsFiles<=0)
            {
                //Load bigrams and trigrams
                NGramTrie nGramTrie=new NGramTrie();
                nGramTrie.loadFromFile(new File(ngramsFile));
                ngramTries.add(nGramTrie);
            }
            else
            {
                for(int i=1; i<=numNGramsFiles; i++)
                {
                    NGramTrie nGramTrie=new NGramTrie();
                    nGramTrie.loadFromFile(new File(ngramsFile+Integer.toString(i)));
                    ngramTries.add(nGramTrie);
                }
            }

            //Load lexical forms
            ChunksTrie chunksTrie=new ChunksTrie(generateAlignments);
            chunksTrie.loadFromFile(new File(lexformsFile));

            int numLine=1;
            BufferedWriter writer = new BufferedWriter( new OutputStreamWriter(System.out,"UTF-8") );
            for(NGramTrie nGramTrie: ngramTries)
            {
                System.err.println("Genrating line "+Integer.toString(numLine)+" of "+Integer.toString(ngramTries.size()));
                ChunksTrie filteredChunks= chunksTrie.filterGivenSources(nGramTrie);
                
                ChunkPhraseGenerator phraseGenerator=new ChunkPhraseGenerator(interchunk,filteredChunks,generateAlignments);
                phraseGenerator.generatePhrases(writer,nGramTrie,ngramOrder);
                numLine++;
            }
            writer.close();
            
        }
        else if (line.hasOption("p"))
        {
            System.err.println("Reverse pretransfer option selected");
            BufferedWriter writer = new BufferedWriter( new OutputStreamWriter(System.out,"UTF-8") );
            InputStreamReader reader= new InputStreamReader(System.in,"UTF-8");

            ReversePreTransfer rpt=new ReversePreTransfer();

            System.err.print("Loading forms split by pretransfer... ");
            rpt.loadFromFile(new File(splitFile));
            System.err.println("OK");

            System.err.print("Processing input... ");
            rpt.reverse(reader, writer,generateAlignments);
            System.err.println("OK");

            reader.close();
            writer.close();
        }

    }
}
