/*
#  -------------------------------------------------------------------
# This file is part of Rule2Phrase.
#
# Rule2Phrase is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Rule2Phrase is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rule2Phrase.  If not, see <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------
*/

package es.ua.dlsi.hybridmt.apertiumphrasegenerator;

import es.ua.dlsi.hybridmt.apertiumphrasegenerator.trie.Node;
import es.ua.dlsi.hybridmt.apertiumphrasegenerator.trie.Trie;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author vmsanchez
 */
public class WordsTrie {

    private Trie<String> trie;
    private Pattern tagPattern=Pattern.compile("<([^>]+)>");

    public WordsTrie() {
        trie= new Trie<String>("");
    }

    public void loadFromFile(File f)
    {
        String line;
        BufferedReader reader=null;
        try
        {
        reader=new BufferedReader(new InputStreamReader(new FileInputStream(f),Charset.forName("UTF-8") ));
            while((line=reader.readLine())!=null)
            {
                processLine(line);
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if(reader!=null)
                try{
                reader.close();
                }catch(Exception e){}

        }
    }

    public List<String> getWordList()
    {
        List<String> wordList=new ArrayList<String>();

        List<List<String>> trieList= trie.getItemList();
        for (List<String> item: trieList)
        {
            if(item.size()>1)
            {
                String lemma=item.remove(item.size()-1);
                item.add(0, lemma);
                wordList.add("^"+Util.join(item, "")+"$");
            }
        }

        return wordList;
    }

    public WordsTrie getSubtrie(List<String> taglist) {
        WordsTrie subwtrie=new WordsTrie();

        boolean strict=true;
        if("*".equals(taglist.get(taglist.size()-1)))
        {
            strict=false;
            taglist=taglist.subList(0, taglist.size()-1);
        }

        Trie<String> subTrie=trie.getSubTrie(taglist, "");
        if (strict && subTrie!=null)
        {
            Node<String> node=subTrie.getNode(taglist);
            Set<String> toRemove= new HashSet<String>();
            for(String key: node.getChildren().keySet())
            {
                if(key.startsWith("<"))
                {
                    toRemove.add(key);
                }
            }
            for(String key:toRemove)
            {
                node.getChildren().remove(key);
            }
        }

        if (subTrie!=null)
            subwtrie.trie=subTrie;
        return subwtrie;
    }

    public void merge(WordsTrie wt)
    {
        this.trie.merge(wt.trie);
    }

    private void processLine(String line)
    {
        String lemma;
        List<String> tags=new LinkedList<String>();
        
        int startTags=line.indexOf("<");
        if (startTags >-1)
        {
            lemma=line.substring(1, startTags);
            Matcher m=tagPattern.matcher(line.substring(startTags));
            while(m.find())
            {
                tags.add(m.group());
            }
            tags.add(lemma);
            trie.insertElement(tags);
        }
    }

    public void pruneLeaves(String lemma) {
        Set<String> allowedLemmas= new HashSet<String>();
        allowedLemmas.add(lemma);
        trie.pruneLeaves(allowedLemmas);
    }
    

}
