#!/usr/bin/python
# coding=utf-8
# -*- encoding: utf-8 -*-

#  -------------------------------------------------------------------
# This file is part of Rule2Phrase.
#
# Rule2Phrase is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Rule2Phrase is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rule2Phrase.  If not, see <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------

import argparse,sys, subprocess, os

class Rule2Phrase:
	
	def is_multilevel_transfer(self):
		filename=self.args.apertium_prefix+"/share/apertium/modes/"+self.args.sl+"-"+self.args.tl+".mode"
		f = open(filename, 'r')
		contents=f.read()
		f.close()
		if contents.find('interchunk') > -1:
			return True
		else:
			return False

	def create_apertium_modes(self):
		command="bash "+self.mypath+"/generateApertiumSpecialModes.sh "+self.args.sl+" "+self.args.tl+" "+self.args.apertium_prefix+"/share/apertium/modes"
		output,error = subprocess.Popen(command, stderr= subprocess.PIPE, shell=True).communicate()
		print >>sys.stderr, error

	def guess_apertium_pair(self):
		dirDefaultOrder=self.args.apertium_src_dir+"/apertium-"+self.args.sl+"-"+self.args.tl
		dirInverseOrder=self.args.apertium_src_dir+"/apertium-"+self.args.tl+"-"+self.args.sl
		if os.path.isdir(dirDefaultOrder):
			return self.args.sl+"-"+self.args.tl
		elif os.path.isdir(dirInverseOrder):
			return self.args.tl+"-"+self.args.sl
		else:
			print >>sys.stderr, "Cannot find directory for neither "+self.args.sl+"-"+self.args.tl+" nor "+	self.args.tl+"-"+self.args.sl
			return self.args.sl+"-"+self.args.tl

	def execute_makefile(self,target,dirsynth,dirngrams,filterfile):
		
		if self.multitransfer:
			dupdetermined="dupdetermined"
			transfer2level="yes"
		else:
			transfer2level="no"
			dupdetermined="no"

		if self.args.discard_multiword_alignments:
			removemwals="--removeMWAlignments"
		else:
			removemwals="''"
		
		command="make -f Makefile-synthetic-general "+target+" DIRSYNTH="+dirsynth+" DIRNGRAMS="+dirngrams+" ALIGNMENTS=yes TRANSFER2LEVEL="+transfer2level+" APERTIUMPAIR="+self.apertium_pair+" DUPDETERMINED="+dupdetermined+" FILTERFILES="+filterfile+" EXTRASCRIPTSDIR="+self.mypath+" NGRAMORDERSYNTH=3 PREFIXFILTERING=no APERTIUMBINDIR="+self.args.apertium_prefix+"/bin APERTIUMPAIRSDIR="+self.args.apertium_src_dir+" SL="+self.args.sl+" TL="+self.args.tl+" MYSCRIPTSDIR=."
		output,error = subprocess.Popen(command, stderr= subprocess.PIPE, shell=True).communicate()
		print >>sys.stderr, error

	def extract_ngrams(self):
		print "Creating Apertium special modes"
		self.create_apertium_modes()

		print "Extracting n-grams"
		self.execute_makefile("onlyngrams",".",self.args.output,self.args.test)

	def gen_phrases(self):
		print "Creating Apertium special modes"
		self.create_apertium_modes()

		print "Generating phrases"
		self.execute_makefile("all",self.args.output,self.args.n_grams,"")
	
	def execute_makefile_integration(self,strategy,phrasesfilename):	
		command="make -f Makefile-mosestrain-apertium-"+strategy+" DIRSYNTH="+self.args.apertium_phrases+" ALIGNMENTS=yes DIR="+self.args.model_dir+" MOSESDIR="+self.args.moses_dir+" SL="+self.args.sl+" TL="+self.args.tl+" MYSCRIPTSDIR=. ORIGINAL_TRAIN="+self.args.training_corpus+" SYNTHFILE="+phrasesfilename+" MOSESSCRIPTSDIR="+self.args.moses_scripts_dir+" LM_FILE="+self.args.lm_file+" LM_ORDER=5 REMOVEMWALS="+removemwals
		output,error = subprocess.Popen(command, stderr= subprocess.PIPE, shell=True).communicate()
		print >>sys.stderr, error
	
	def get_apertium_phrases_filename(self):
		if os.path.isfile(self.args.apertium_phrases+'/syntheticCorpus2levelplusdetdict.'+self.args.sl):
			return 'syntheticCorpus2levelplusdetdict'
		elif os.path.isfile(self.args.apertium_phrases+'/syntheticcorpusplusdetdict.'+self.args.sl):
			return 'syntheticcorpusplusdetdict'
		else:
			print >>sys.stderr, "Cannot find Apertium-generated phrases."
			return 'syntheticcorpusplusdetdict'
	
	def build_SMT(self,strategy):
		print "Building hybrid SMT system"
		filename=self.get_apertium_phrases_filename()
		self.execute_makefile_integration(strategy,filename)

	def patch_apertium(self):
		print "Patching Apertium source code"
		command="cd "+self.args.apertium_src_dir+"/apertium; patch -p0 < "+self.mypath+"/apertiumAlignmentsPatch.diff"
		output,error = subprocess.Popen(command, stderr= subprocess.PIPE, shell=True).communicate()
		print >>sys.stderr, error
		print "Apertium source code patched. Now you should compile it and install it again."

	def doall(self,myprogram,myargs):
		parser = argparse.ArgumentParser(description='Extracts a set of bilingual phrase pairs from the Apertium linguistic resources and integrates them into the translation models built with Moses.')
		parser.add_argument('--extract-n-grams',action='store_true',help='Extract n-grams from a source language test. Should be done before generating phrase pairs from the Apertium data. Only phrase pairs containing solely n-grams from this set will be generated in the next step.')
		parser.add_argument('--gen-phrases',action='store_true',help='Generate bilingual phrase pairs from the Apertium linguistic resources. Only phrase pairs whose source side contains solely the n-grams from the list  referenced by the option \'--n-grams\' are generated.')
		parser.add_argument('--test',help='ONLY WITH FLAG \'--extract-n-grams\'. Text to extract n-grams from.' )
		parser.add_argument('--n-grams',help='ONLY WITH FLAG \'--gen-phrases\'. Directory containing the n-grams for filtering.' )
		parser.add_argument('--sl',help='Source language code')
		parser.add_argument('--tl',help='Target language code')
		parser.add_argument('--output',help='ONLY WITH FLAGS \'--extract-n-grams\' or \'--gen-phrases\'. Directory where the extracted n-grams or the generated phrase pairs will be stored.')
		parser.add_argument('--apertium-prefix',default='/usr/local', help='ONLY WITH FLAGS \'--extract-n-grams\' or \'--gen-phrases\'. System prefix where Apertium is installed. Apertium executable must be in \'APERTIUM_PREFIX/bin\'.')
		parser.add_argument('--apertium-src-dir',default='/usr/local/share/src/apertium', help='ONLY WITH FLAG \'--gen-phrases\'. Directory where the source code of the different Apertium modules and language pair packages can be found.')
		parser.add_argument('--build-SMT',help='Build a PBSMT system using a parallel corpus and the phrase pairs generated from the Apertium linguistic resources. Three strategies can be followed: adding the phrase pairs to the training corpus (using the argument \'corpus-rules\' for thihs option), adding them directly to the phrase table (using the argument \'phrase-rules\' for thihs option) or adding them to both (using the argument \'pc-rules\' for thihs option).')
		parser.add_argument('--apertium-phrases', help='ONLY WITH FLAG \'--build-SMT\'. Directory where the Apertium-generated phrase pairs are stored.')
		parser.add_argument('--training-corpus',help='ONLY WITH FLAG \'--build-SMT\'. Parallel corpus used to train the PBSMT system. The system will search for the files \'TRAINING_CORPUS.SL\' and \'TRAINING_CORPUS.TL\'')
		parser.add_argument('--lm-file', help='ONLY WITH FLAG \'--build-SMT\'. Language model file.')
		parser.add_argument('--moses-dir', default='/usr/local/lib/moses',help='ONLY WITH FLAG \'--build-SMT\'. Moses installation directory. Default value is \'/usr/local/lib/moses\'.')
		parser.add_argument('--moses-scripts-dir', default='/usr/local/lib/moses-scripts',help='ONLY WITH FLAG \'--build-SMT\'. Moses scripts directoru. Default value is \'/usr/local/lib/moses-scripts\'.')
		parser.add_argument('--model-dir', default='~/hybrid-model',help='ONLY WITH FLAG \'--build-SMT\'. Deirectory where the PBSMT models resulting from the training process will be stored. Default value is \'~/hybrid-model\'.')
		parser.add_argument('--discard-multiword-alignments',action='store_true', help='ONLY WITH FLAG \'--build-SMT\'. Discard alignments from multi-word units.')

		parser.add_argument('--patch-apertium',action='store_true')

		args = parser.parse_args(myargs)
		
		self.mypath=os.path.abspath(os.path.dirname(myprogram))
		self.args=args
	
		if args.extract_n_grams or args.gen_phrases:
			multitransfer=self.is_multilevel_transfer()
			self.multitransfer=multitransfer
			self.apertium_pair=self.guess_apertium_pair()

#		print self.apertium_pair

		if args.extract_n_grams:
			self.extract_ngrams()
		elif args.gen_phrases:
			self.gen_phrases()
		elif args.build_SMT:
			strategy=args.build_SMT
			self.build_SMT(strategy)
		elif args.patch_apertium:
			self.patch_apertium()

rule2phrase=Rule2Phrase()
rule2phrase.doall(sys.argv[0],sys.argv[1:])

